#!/usr/bin/perl
###############################################################
#                  sys_alive.pl  v1.5.3                       #
###############################################################
#    Programme de surveillance de serveur web sous Linux      #
#               par Twidi (scripts@twidi.com)                 #
###############################################################
# Ce programme est libre, vous pouvez le redistribuer et/ou   #
# le modifier selon les termes de la Licence Publique         #
# Générale (GPL) GNU publiée par la Free Software Foundation. #
#                                                             #
# Ce programme est distribué car potentiellement utile, mais  #
# SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris  #
# les garanties de commercialisation ou d'adaptation dans un  #
# but spécifique. Reportez-vous à la Licence Publique         #
# Générale (GPL) GNU pour plus de détails.                    #
#                                                             #
#         -> http://www.april.org/gnu/index.html <-           #
#                                                             #
###############################################################
# Dernière version sur : http://scripts.twidi.com/            #
# Informations : http://b.twidi.com/index.php/tag/sys_alive   #
###############################################################
# Contributeurs :                                             #
# Twidi (scripts@twidi.com) : Programme de base               #
# eB (fdl@ebusiness.be) : Base de la v 1.2.4                  #
# Et vous ?                                                   #
###############################################################
# Historique :                                                #
# v 1.5.3: 2008-09-23 : ajout d'alertes sur la ram utilisee   #
# v 1.5.2: 2007-11-26 : meilleure gestion du brute stop       #
#                       et on ne kill pas sys_alive !!        #
# v 1.5.1: 2007-11-21 : mode "demon" utilisable en tant que   #
#                       script de démarrage (/etc/init.d)     #
# v 1.5  : 2007-11-19 : mode "demon", tentatives supll.       #
#                       d'arrets de demons                    #
# v 1.2.8: 2007-05-07 : Séparation rép conf, data & log       #
#                       + affichage "ps faux" au lieu de top  #
#                       par défaut (modifiable en conf)       #
# v 1.2.7: 2006-08-22 : Correction gestion mémoire pour       #
#                       compatibilité noyaux 2.4 et 2.6       #
# v 1.2.6: 2005-03-13 : Ajoute de la possibilité d'ignorer    #
#                       temporairement un démon en créant     #
#                       (par exemple via touch) un fichier    #
#                       ignore_nomdudemon dans le répertoire  #
#                       du fichier de configuration           #
# v 1.2.5: 2003-12-16 : Ajout du test de la variable d'env.   #
#                       "TERM", et présence des secondes dans #
#                       le top, et use strict (pas de -w a    #
#                       cause de warnings type $main::)       #
#                       + diverses petites modifications      #
# v 1.2.4: 2002-11-18 : Ajout de l'activité apache et mysql,  #
#                       + choix du fichier log + possibilité  #
#                       d'interdire de tuer les process root  #
# v 1.2.3: 2002-02-17 : Ajout du test sur le user des demons  #
# v 1.2.2: 2002-02-07 : Correction bug sur l'envoi du mail    #
# v 1.2.1: 2002-02-01 : Modification de la façon d'écrire     #
#                       dans le fichier de données, et mise   #
#                       en log du résultats des appels        #
#                       système (arrêts, relances...)         #
# v 1.2  : 2002-01-30 : Externalisation des informations de   #
#                       configuration (sys_alive.conf) +      #
#                       fichier de log + corrections mineures #
# v 1.1  : 2002-01-25 : Paramètres stop et start + 2eme       #
#                       niveau d'alerte avec arrêt de démons  #
# v 1.0.1: 2002-01-22 : License GPL                           #
# v 1.0  : 2002-01-16 : Première version                      #
###############################################################

###############################################################
#    CE PROGRAMME DOIT ETRE INSTALLE ET EXECUTE EN ROOT       #
#             et doit être exécutable (chmod 755)             #
#-------------------------------------------------------------#
# Installation :                                              #
# 1) copier sys_alive.pl et sys_alive.conf dans un répertoire #
# 2) adapter la configuration dans le fichier sys_alive.conf  #
# 3) mettre la configuration en chmod 600                     #
# 4) mettre le script en cron(4.1) ou le lancer en demon(4.2) #
# 4.1) mettre le script en cron (en root) toutes les minutes: #
#    Mettre en conf mode_demon.actif à 0                      #
#    Lancer, en root : crontab -e                             #
#    Et ajouter : * * * * * /chemin/de/sys_alive/sys_alive.pl #
# 4.2) lancer le script en demon                              #
#    Mettre en conf mode_demon.actif à 1                      #
#    Le lancer : /chemin/de/sys_alive/sys_alive.pl start      #
#    Le lier par exemple vers /etc/init.d/sys_alive et mettre #
#    à jour les scripts de démarrage pour un lancement        #
#    automatique (en fonction de votre distribution)          #
###############################################################

use strict;

use Proc::Daemon;
use File::Pid;

#------------------------------------------------------------------------------#
# répertoire des données : remplacer *éventuellement* GetLocalRep() par
# le répertoires où est stocké le fichier de configuration, uniquement s'il ne
# doit pas être stocké dans le même répertoire que ce programme ou si sys_alive
# est configuré pour tourner en mode démon

my $rep_conf = GetLocalRep();
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
####  CORPS DU PROGRAMME  ####
#------------------------------------------------------------------------------#
# fichier de configuration
die 'Fichier conf non trouvé' if ! -e $rep_conf.'/sys_alive.conf';
require($rep_conf.'/sys_alive.conf');

# autres répertoires
$main::rep_data = $rep_conf if !$main::rep_data;
$main::rep_log  = $rep_conf if !$main::rep_log;

# variables globales
my $version = '1.5.3';
my $arg = lc(shift(@ARGV));
if (!$main::logfile) { $main::logfile = $main::rep_log.'/sys_alive.log'; }
my $datafile = $main::rep_data.'/sys_alive.dat';
my %infos = LireFichier();
my %ignore;
my %pid = (
	obj   => undef,
	file  => $main::mode_demon{'pid_file'} || '/var/run/sys_alive.pid',
	value => 0,
);

my $demon_running = demon_running();
my $mode_demon_actif = $main::mode_demon{'actif'} || $demon_running;

my $term = $ENV{'TERM'} ? '' : 'TERM=vt100; export TERM;'; # on simule un terminal si la variable d'environnement n'est pas définie

# on arrête ou on relance sys_alive si demandé
if ($arg eq 'stop') {
	SetInfo('stopped', 1);
	if ($mode_demon_actif) {
		$pid{'obj'} = File::Pid->new({
			file => $pid{'file'},
			pid  => $demon_running,
		});
		$pid{'value'} = $demon_running;
	} else {
		WriteLog('Désactivation de sys_alive');
	}
} elsif ($arg eq 'start') {
	SetInfo('stopped', 0);
	WriteLog('Réactivation de sys_alive') if !$mode_demon_actif;
} elsif ($mode_demon_actif) {
	WriteLog('Usage en mode demon : /chemin/de/sys_alive/sys_alive.pl start|stop', 1);
}

exit if $infos{'stopped'};

if ($mode_demon_actif) {
	mode_demon();
} else {
	sys_alive();
}

# mode demon
sub mode_demon {
	WriteLog('Erreur : Sys_alive est déjà lancé en mode démon (pid='.$demon_running.')', 1) if $main::mode_demon{'actif'} and $demon_running;

	Proc::Daemon::Init;
	$pid{'obj'} = File::Pid->new({
		file => $pid{'file'},
		pid  => $$,
	});

	if ($pid{'value'} = $pid{'obj'}->write()) {
		WriteLog('Sys_alive lancé en mode démon (pid='.$pid{'value'}.')');
	} else {
		WriteLog('Erreur : Problème en écrivant le fichier pid : '.$@, 1) if $@;
	}

	while(1) {
		%infos = LireFichier();
		last if $infos{'stopped'};
		sys_alive();
		sleep($main::mode_demon{'wait'} || 60);
	}
} # fin de sub mode_demon


# fonction principale
sub sys_alive {

	# variables locales
	my @alertes;
	my $alertes_niveau2 = 0;
	my $alerte_importante = 0;
	my %data;
	my @killed_process;
	my @restarted_demons;
	my %process;

	# on ne vas gérer le tout que si sys_alive n'est pas stoppé
	if (!$infos{'stopped'}) {
		# récupération des données
		my %datetime = GetTime();
		($data{'ram_physique'}, $data{'ram_reelle'}, $data{'ram_swap'}) = GetInfosRam();
		($data{'cpu_1'}, $data{'cpu_5'}, $data{'cpu_15'}, $data{'nb_process'}) = GetInfosCPU();
		my $top = `$term top b n 1`; # récupération des process
		my $free = `free -t`;

		###### gestion des alertes
		if ($main::gestion{'seuil_alerte'}) {

			# gestion de l'alerte swap
			if ($main::seuil_alertes{'swap'}) {
				if ($data{'ram_swap'} > $main::seuil_alertes{'swap'}) { # en cas de dépassement
					push @alertes, 'swap';
					WriteLog("Seuil 1 du swap dépassé : $data{'ram_swap'} > $main::seuil_alertes{'swap'}");
				} # fin de if data > seuil
			} # fin de if swap

			# gestion de l'alerte ram_used
			if ($main::seuil_alertes{'ram_used'}) {
				if ($data{'ram_reelle'} > $main::seuil_alertes{'ram_used'}) { # en cas de dépassement
					push @alertes, 'ram_used';
					WriteLog("Seuil 1 de la ram réelle dépassé : $data{'ram_reelle'} > $main::seuil_alertes{'ram_used'}");
				} # fin de if data > seuil
			} # fin de if ram_used

			# gestion de l'alerte load_average
			if ($main::seuil_alertes{'load_average'}) {
				if ($data{'cpu_1'} > $main::seuil_alertes{'load_average'}) { # en cas de dépassement
					push @alertes, 'load_average';
					WriteLog("Seuil 1 du load average dépassé : $data{'cpu_1'} > $main::seuil_alertes{'load_average'}");
				} # fin de if data > seuil
			} # fin de if load_average

			# gestion de l'alerte nb_process
			if ($main::seuil_alertes{'nb_process'}) {
				if ($data{'nb_process'} > $main::seuil_alertes{'nb_process'}) { # en cas de dépassement
					push @alertes, 'nb_process';
					WriteLog("Seuil 1 du nb de process dépassé : $data{'nb_process'} > $main::seuil_alertes{'nb_process'}");
				} # fin de if data > seuil
			} # fin de if nb_process

		} # fin de if gestion seuil_alerte

		#### si on a enregistré des alertes, avant les kills éventuels on va récupérer certaines valeurs du système
		my ($mysql_activity_msg, $apache_activity_msg);
		if (@alertes+0) {
			if ($main::mysql_activity) {	$mysql_activity_msg = Execute($main::mysql_activity, 1); }
			if ($main::apache_activity) {	$apache_activity_msg = Execute($main::apache_activity, 1); }
		} # fin de if alertes


		###### gestion des alertes de niveau 2
		if ($main::gestion{'stop_alerte'}) {

			# gestion de l'alerte swap
			if ($main::seuil_alertes_niveau2{'swap'}) {
				if ($data{'ram_swap'} > $main::seuil_alertes_niveau2{'swap'}) { # en cas de dépassement
					if (!$alertes_niveau2) { unshift @alertes, '--ALERTE--'; }
					$alertes_niveau2++;
					$alerte_importante = 1;
					WriteLog("Seuil 2 du swap dépassé : $data{'ram_swap'} > $main::seuil_alertes_niveau2{'swap'}");
				} # fin de if data > seuil
			} # fin de if swap

			# gestion de l'alerte ram_used
			if ($main::seuil_alertes_niveau2{'ram_used'}) {
				if ($data{'ram_reelle'} > $main::seuil_alertes_niveau2{'ram_used'}) { # en cas de dépassement
					if (!$alertes_niveau2) { unshift @alertes, '--ALERTE--'; }
					$alertes_niveau2++;
					$alerte_importante = 1;
					WriteLog("Seuil 2 de la ram reelle dépassé : $data{'ram_reelle'} > $main::seuil_alertes_niveau2{'ram_used'}");
				} # fin de if data > seuil
			} # fin de if ram_used

			# gestion de l'alerte load_average
			if ($main::seuil_alertes_niveau2{'load_average'}) {
				if ($data{'cpu_1'} > $main::seuil_alertes_niveau2{'load_average'}) { # en cas de dépassement
					if (!$alertes_niveau2) { unshift @alertes, '--ALERTE--'; }
					$alertes_niveau2++;
					$alerte_importante = 1;
					WriteLog("Seuil 2 du load average dépassé : $data{'cpu_1'} > $main::seuil_alertes_niveau2{'load_average'}");
				} # fin de if data > seuil
			} # fin de if load_average

			# gestion de l'alerte nb_process
			if ($main::seuil_alertes_niveau2{'nb_process'}) {
				if ($data{'nb_process'} > $main::seuil_alertes_niveau2{'nb_process'}) { # en cas de dépassement
					if (!$alertes_niveau2) { unshift @alertes, '--ALERTE--'; }
					$alertes_niveau2++;
					$alerte_importante = 1;
					WriteLog("Seuil 2 du nb de process dépassé : $data{'nb_process'} > $main::seuil_alertes_niveau2{'nb_process'}");
				} # fin de if data > seuil
			} # fin de if nb_process

			# si on est en alerte niveau 2, on va stopper les démons à stopper
			if ($alertes_niveau2) {
				foreach my $demon (keys(%main::demons_to_stop)) { # pour chaque demon
				if (!IgnoreDemon($demon)) { # si on ne doit pas l'ignorer
					my $canstop = ($top =~ /\s$demon\s*$/m); # on arrete le demon s'il est lancé
					SetInfo('skip_demon_'.$demon, time(), 1); # on met le demon dans la liste des démons à ne pas relancer
					if ($canstop) {
						WriteLog("Alerte de niveau 2 : arrêt de $demon pour $main::demons_to_stop{$demon} secondes");
						Execute($main::demons_stop{$demon});
						# essais supplémentaires si encore des traces
						if (my $max_brute = $main::demons_brute_stop{$demon}) {
							my $essai = 1;
							while ((my $top2 = `$term top b n 1`) =~ /\s$demon\s*$/m) {
								WriteLog("Trop d'essai pour forcer l'arrêt de $demon qui semble tourner encore"), last if $essai > $max_brute;
								my @pids; map { push @pids, $1 if /^\s*(\d+).+\s$demon\s*$/} split(/\n/, $top2);
								if (@pids+0) {
									WriteLog("Suite alerte de niveau 2 : force arrêt de $demon, essai $essai (kill @pids)");
									Execute('kill -9 '.join(' ', @pids));
								}
								$essai++;
							} # fin de while
						} # fin de if max brute
					} else {
						WriteLog("Alerte de niveau 2 : impossible de stopper $demon car il n'est pas lancé");
					}
				} # fin de if !ignore
				} # fin de foreach
			} # fin de if

		} # fin de if gestion stop_alerte

		###### gestion des process trop longs
		if ($main::gestion{'kill_process'} or $main::gestion{'reload_demons'}) {

			# préparation du top à être interprété
			my @top = split(/\n/, $top);
			splice(@top, 0, 7);
			chomp(@top);
			# travail sur chaque ligne
			foreach my $ligne (@top) {
				if ($ligne =~ /(\d+)\s*(\w*)\s*.*?((\d*):(\d*\.?\d*)) (.*?)\s*$/) {
					my %info_process = (
						pid => $1,
						user => $2,
						duree => $3,
						minutes => $4,
						secondes => $5,
						process => $6,
					);
					# recherche des demons à relancés
					if ($main::gestion{'reload_demons'}) {
						# on enregistre le fait que ce process est lancé
						$process{$info_process{'process'}}->{'present'} = 1;
						# et l'utilisateur concerné
						$process{$info_process{'process'}}->{$info_process{'user'}} = 1;
					} # fin de gestion reload demons

					# recherche des process à killer
					if ($main::gestion{'kill_process'}) {
						$info_process{'nb_sec'} = $info_process{'minutes'} * 60 + $info_process{'secondes'}; # calcul du nb de secondes
						# en cas de process trop long
						if ($info_process{'nb_sec'} > $main::delai_max_process) {
							my $to_del = 0;
							# on regarde s'il est à tuer
							foreach my $to_kill (@main::process_to_kill) { if ($info_process{'process'} =~ /$to_kill/) { $to_del = 1; } }
							# on regarde s'il est à garder
							foreach my $to_keep (@main::process_to_keep) { if ($info_process{'process'} =~ /$to_keep/) { $to_del = 0; } }
							# on ne tue le process root que si autorisé
							if (($info_process{'user'} eq 'root') and (!$main::kill_root_process)) {	$to_del = 0; }
							# on tue le process...
							if ($to_del) { # s'il est à tuer
								push @killed_process, \%info_process; # on sauvegarde l'info
								$alerte_importante = 1; # pour forcer l'envoie de mail quelque soit le délai entre les mails
								WriteLog("Process $info_process{'process'} tué car trop long (durée=$info_process{'duree'}, pid=$info_process{'pid'}, user=$info_process{'user'})");
								Execute("kill -9 $info_process{'pid'}"); # on kill le process
							} # fin de if to_del
						} # fin de if info_process > alerte
					} # fin de if kill_process
				} # fin de if ligne =~
			} # fin de foreach
			if (@killed_process+0 > 0) { push @alertes, 'kill'; }
		} # fin de if gesion kill_process or gestion reload_demons

		###### gestion des demons
		if ($main::gestion{'reload_demons'}) {
			# on va gerer chaque demon
			foreach my $demon (keys(%main::demons_start)) { if (defined($demon)) {
				if (!IgnoreDemon($demon)) { # si on ne doit pas l'ignorer
					# check du demon
					my $present = ($process{$demon}->{'present'} and (defined($main::demons_user{$demon}) ? $process{$demon}->{$main::demons_user{$demon}} : 1));
					# si le demon n'est pas présent et qu'il n'est pas à l'état "skip"
					if (!$present and ($infos{'skip_demon_'.$demon} + $main::demons_to_stop{$demon} < time())) {
						push @restarted_demons, $demon; # sauvegarde de l'information
						$alerte_importante = 1; # pour forcer l'envoie de mail quelque soit le délai entre les mails
						if ($infos{'skip_demon_'.$demon}) {
							WriteLog("Démon $demon relancé, durée de $main::demons_to_stop{$demon} secondes écoulée");
						} else {
							WriteLog("Démon $demon relancé car absent");
						}
						SetInfo('skip_demon_'.$demon, 0); # on annule l'état "skip" s'il y etait
						Execute($main::demons_start{$demon}); # on relance le demon
					} #fin de if
				} # fin de if !ignore
			}} # fin de foreach demon if defined
			if (@restarted_demons+0 > 0) { push @alertes, 'demon'; }
		} # fin de if gesion reload_demons

		###### envoi du mail (si alerte importante ou ecart entre 2 mails respecté)
		if (!$alerte_importante and ((@alertes+0 > 0) and ($datetime{'nb_sec'} - $infos{'date_msg'} < $main::ecart_alerte))) {
			WriteLog(" -> Aucun mail envoyé car le dernier date de moins de $main::ecart_alerte secondes");
		}
		if ($alerte_importante or ((@alertes+0 > 0) and ($datetime{'nb_sec'} - $infos{'date_msg'} >= $main::ecart_alerte))) {
			# alerte de niveau 2
			my ($alerte2, $alerte2sms);
			if ($alertes_niveau2) {
				$alerte2 = "\nIMPORTANT : Une alerte de niveau 2 a été détectée. Les démons suivants ont été arrétés :\n";
				$alerte2sms = "Démons tués:";
				foreach my $demon (keys(%main::demons_to_stop)) { # pour chaque demon
					if (!IgnoreDemon($demon)) { # si on ne doit pas l'ignorer
						$alerte2 .= $demon.' ('.$main::demons_to_stop{$demon}.' secondes.) - ';
						$alerte2sms .= "$demon,";
					} # fin de if !ignore
				} # fin de foreach
				$alerte2 .= "\n";
			}
			# composition du message
			my $proc_activity_msg = $main::proc_activity ? Execute($main::proc_activity, 1) : $top;
			my $msg_mail = <<"ENDMAIL";

Alerte du $datetime{'date'} à $datetime{'hour'}h$datetime{'min'}
$alerte2
Données :
---------
Mem. physique : $data{'ram_physique'}
Mem. réelle   : $data{'ram_reelle'}
Mem. Swap     : $data{'ram_swap'}

Load average : $data{'cpu_1'} / $data{'cpu_5'} / $data{'cpu_15'}
Nb process   : $data{'nb_process'}

Informations RAM  :
-------------------
$free

Informations CPU :
------------------
$proc_activity_msg
ENDMAIL

		my $msg_sms = <<"ENDSMS";

$datetime{'date'} $datetime{'hour'}h$datetime{'min'}

Ram : $data{'ram_physique'}/$data{'ram_reelle'}/$data{'ram_swap'}

Load : $data{'cpu_1'}/$data{'cpu_5'}/$data{'cpu_15'}

Proc : $data{'nb_process'}
ENDSMS

			# ajout des infos sur les process tués
			if (@killed_process+0 > 0) {
				$msg_mail .= "\nListe des process tués :\n------------------------\n";
				my $nb_process_killed = @killed_process+0;
				$msg_sms .= "\nKilled: ";
				foreach my $process (@killed_process) { # pour chaque process
					$msg_mail .= " - Process : $process->{'process'}, User : $process->{'user'}, PID : $process->{'pid'}, Durée : $process->{'minutes'}:$process->{'secondes'}\n";
					$msg_sms .= "$process->{'process'}($process->{'user'}),";
				} # fin de foreach process
			} # fin de if @killed_process

			# ajout des infos sur les démons relancés
			if (@restarted_demons+0 > 0) {
				$msg_mail .= "\nListe des démons relancés :\n-----------------------------\n";
				$msg_sms .= "\nDemons: ";
				foreach my $demon (@restarted_demons) { # pour chaque démon
					$msg_mail .= " - $demon\n";
					$msg_sms .= "$demon,";
				} # fin de foreach demon
				$msg_sms = substr($msg_sms, 0, length($msg_sms)-1);
			} # fin de if @restarted_demons

			# ajout de l'alerte 2 pour les sms
			$msg_sms .= "\n".$alerte2sms;

			# 20 dernières lignes de log
			if ($main::joint_log) {
				sleep(10);
				my $tail_log = qx(tail -n20 $main::logfile);
				$msg_mail .= "\nLog :\n-----\n".$tail_log;
			}


			if ($mysql_activity_msg) {
				$msg_mail .= <<"ENDMAIL";


Informations MySQL :
--------------------
$mysql_activity_msg
ENDMAIL
			}

			if ($apache_activity_msg) {
				$msg_mail .= <<"ENDMAIL";


Informations Apache :
---------------------
$apache_activity_msg
ENDMAIL
			}

			$msg_mail .= <<"ENDFINMAIL";

--
Mail envoyé par sys_alive.pl, programme de surveillance serveur v$version
http://scripts.twidi.com/ ou http://b.twidi.com/index.php/tag/sys_alive
ENDFINMAIL

			# composition du sujet
			my $sujet = 'Alerte';
			foreach (@alertes) { $sujet .= ' '.$_; };

			# envoi du mail
			my %mail = (
				sujet => $sujet,
				msg => $msg_mail,
			);
			foreach (@main::mails) {
				$mail{'to'} = $_;
				SendMail(%mail);
				WriteLog("Envoi d'un mail à $mail{'to'}");
			}

			# envoi du sms
			my %sms = (
				sujet => $sujet,
				msg => $msg_sms,
			);
			foreach (@main::mails_sms) {
				$sms{'to'} = $_;
				SendMail(%sms);
				WriteLog("Envoi d'un mail format sms à $sms{'to'}");
			}

			# enregistrement de la date de dernier envoie de mail
			SetInfo('date_msg', time(), 1);

		} # fin de if ... (envoi mail)

	} # fin de if (!$infos{'stopped'}) {

} # fin de sub sys_alive

#------------------------------------------------------------------------------#
####  FONCTIONS  ###
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# GetTime : Récupère la date et l'heure et renvoie un hash avec les éléments
#------------------------------------------------------------------------------#
sub GetTime {
	my($time) = @_;
	if (!$time) { $time = time; }

	my(%dt);
	$dt{'nb_sec'} = $time;

	($dt{'dayinyear'}, $dt{'year'}, $dt{'month'}, $dt{'day'}, $dt{'hour'}, $dt{'min'}, $dt{'sec'}) = (localtime($time))[7,5,4,3,2,1,0];

	$dt{'month'}++;
	$dt{'year'} += 1900;
	if ($dt{'month'} > 12) { $dt{'month'} = 0; }

	foreach (qw(sec min hour day month)) { $dt{$_} = sprintf("%02d", $dt{$_}); }

	$dt{'date'} = sprintf("%04d-%02d-%02d",$dt{'year'},$dt{'month'},$dt{'day'});
	$dt{'time'} = sprintf("%02d:%02d:%02d",$dt{'hour'},$dt{'min'},$dt{'sec'});

	return %dt;
}
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# GetInfosRam : Renvoie la ram physique, la ram reellement prise et le swap
#------------------------------------------------------------------------------#
sub GetInfosRam {
	my %infos;
	open (F,"/proc/meminfo");
	while (<F>) {
		$infos{$1} = $2 if /^(\w+):\s*(\d+) kB$/;
	}
	close(F);

	my $ram_physique = $infos{'MemTotal'} - $infos{'MemFree'};
	my $ram_reelle = $ram_physique - $infos{'Buffers'} - $infos{'Cached'};
	my $ram_swap = $infos{'SwapTotal'} - $infos{'SwapFree'};

	return ($ram_physique, $ram_reelle, $ram_swap);
}
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# GetInfosCPU : Renvoie le load average sur 1 mn, 5 mn et 15 mn, et  le nb de process en cours
#------------------------------------------------------------------------------#
sub GetInfosCPU {
	open (F,"/proc/loadavg"); my $loadavg=<F>; close(F);
	my ($cpu_1, $cpu_5, $cpu_15, $temp1, $temp2) = split(/\s+/, $loadavg);
	my ($temp3, $nb_process) = split(/\//, $temp1);

	return ($cpu_1, $cpu_5, $cpu_15, $nb_process);
}
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# LireFichier => Lit le fichier de données
#------------------------------------------------------------------------------#
sub LireFichier {
	my($key, $val, %infos);
	open(FILE, "<".$datafile);   # Ouverture du fichier
	my @tab = <FILE>;            # Lecture
	close(FILE);                 # Fermeture

	foreach  (@tab) {                      # Pour chaque ligne (donc, pour chaque paire)
		chop;                              # Retirer le caractère de fin de ligne
		($key, $val) = split(/=/, $_, 2);
		$infos{$key} = $val;               # Et faire la séparation
	}

	return %infos
}
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# EcrireFichier => Ecrit le fichier de données
#------------------------------------------------------------------------------#
sub EcrireFichier {
	my(%infos) = @_;

	LockFichier($datafile, 1);        # On interdit le ficher en écriture (pour sys_alive)
	open(FILE, ">".$datafile);        # Ouverture du fichier
	foreach (keys(%infos)) {          # Pour chaque paire
		print FILE "$_=$infos{$_}\n"; # l'écrire avec un \n
	}
	close(FILE);                      # Fermeture
	LockFichier($datafile, 0);        # On autorise de nouveau l'écriture
}
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# SendMail => Envoie un mail
#------------------------------------------------------------------------------#
sub SendMail {
	my(%mail) = @_;
	open(MAIL,"|$main::sendmail $mail{'to'}");
	print MAIL "To: $mail{'to'}\n";
	print MAIL "From: $main::from\n";
	print MAIL "Subject: $mail{'sujet'}\n";
	print MAIL "$mail{'msg'}\n";
	close (MAIL);
}
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# WriteLog => Ajoute une ligne dans le fichier de log
#------------------------------------------------------------------------------#
sub WriteLog {
	my ($texte, $die) = @_;
	print $texte."\n" if $die;
	LockFichier($main::logfile, 1);
	my %d = GetTime();
	my $ligne = $d{'date'}.' '.$d{'time'}.' > '.$texte."\n";
	open(LOG, '>>'.$main::logfile);
	print LOG $ligne;
	close(LOG);
	LockFichier($main::logfile, 0);
	exit 1 if $die;
	return 1;
}
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# GetLocalRep => Récupération du répertoire de données
#------------------------------------------------------------------------------#
sub GetLocalRep {
	use FindBin qw($Bin);
	return $Bin;
}
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# SetInfo => Met l'information indiquée dans le fichier de données
# on met $time à 1 si $valeur est une durée en secondes et que l'on ne veut
# mettre à jour l'info que si $valeur est supérieur à celle déjà dans le fichier
# de données
#------------------------------------------------------------------------------#
sub SetInfo {
	my ($nom, $valeur, $time) = @_;
	%infos = LireFichier();
	if (!$time or ($valeur > $infos{$nom})) { $infos{$nom} = $valeur; }
	EcrireFichier(%infos);
}
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# Execute => Lance un process et écrit le résultat en log
#------------------------------------------------------------------------------#
sub Execute {
	my($cmd, $no_log) = @_;
	my $result;
	if ($no_log) {
		$result = `$term $cmd`;
	} else {
		LockFichier($main::logfile, 1);
		my $resultat = system($term.' '.$cmd.' >>'.$main::logfile);
		LockFichier($main::logfile, 0);
		$result = '';
	}
	return $result;
}
#------------------------------------------------------------------------------#

#------------------------------------------------------------------------------#
# IgnoreDemon => Indique si on doit ignorer un démon
#------------------------------------------------------------------------------#
sub IgnoreDemon {
	my ($demon) = @_;
	if (!exists($ignore{$demon})) {
		$ignore{$demon} = (-e $rep_conf.'/ignore_'.$demon) ? 1 : 0;
		WriteLog('Démon '.$demon.' ignoré') if $ignore{$demon};
	}
	return $ignore{$demon};
}
#------------------------------------------------------------------------------#


# =================================================================
# LockFichier => Verrou de fichier
#
#- fichier => nom du fichier à locker
#- lock    => à 1 si on lock, à 0 pour libérer
#-
#- Le système consiste à créer un fichier temporaire, correspondant au fichier
#- passé en paramètre auquel on ajoute ".lock"
#- Si ce fichier existe, c'est que le fichier est locké, sinon, il est libre
#- Et si le fichier est locké, on attend sa libération
# =================================================================
sub LockFichier {
	# Récupération des paramètres
	my($Fichier, $Lock) = @_;

	# Création du nom du fichier temporaire.
	my($FichierLock) = $Fichier;
	$FichierLock .= ".lock";

	# En cas de vérouillage
	if ($Lock==1) {

		my($time);
		# Si ce fichier de lock existe, on lit l'heure enregistrée
		if (-e $FichierLock) {
			open(LOCKFILE, "<".$FichierLock);
			$time = <LOCKFILE>;
			close(LOCKFILE);
		}

		# Tant que le fichier lock existe,
		while (-e $FichierLock) {
			# Soit le temps maximum est atteint et le fichier est libéré
			if (time > ($time+30)) { LockFichier($Fichier, 0); }
			# Soit on attend un dixième de seconde avant de re-tester
			select(undef, undef, undef, 0.1);
		}

		# Dès qu'il n'existe plus, on le crée
		open(LOCKFILE, ">".$FichierLock);
		print LOCKFILE time;
		close(LOCKFILE);


	# Et en cas de dévérouillage
	} else {

		# Quand on a fini avec le fichier, on le dévérouille, en supprimant le fichier lock
		unlink($FichierLock);
	}
}
# =================================================================

# test si un demon sys_alive tourne deja
sub demon_running {
	$pid{'obj'} = File::Pid->new({
		file => $pid{'file'},
		pid  => $$,
	});

	my $current_pid;
	if (-e $pid{'file'}) {
		eval { $current_pid = $pid{'obj'}->running(); };
		WriteLog('Erreur : Problème en accédant au fichier pid : '.$@, 1) if $main::mode_demon{'actif'} and $@;
	}

	return $current_pid;
} # fin de demon_running

# END : appelé à la fin du script : supprime le fichier pid si besoin
sub END {
	if ($mode_demon_actif and $pid{'obj'} and $pid{'value'}) {
		$pid{'obj'}->remove();
		Execute('kill -15 '.$pid{'value'}) if $pid{'value'} != $$;
		WriteLog('Sys_alive stoppé en mode démon (pid='.$pid{'value'}.')');
	}
} # fin de END
